@@ -59,8 +59,8 @@
 (ns com.puppetlabs.puppetdb.command
   (:require [clojure.tools.logging :as log]
             [com.puppetlabs.puppetdb.scf.storage :as scf-storage]
-            [com.puppetlabs.puppetdb.catalog :as cat]
-            [com.puppetlabs.puppetdb.report :as report]
+            [com.puppetlabs.puppetdb.catalogs :as cat]
+            [com.puppetlabs.puppetdb.reports :as report]
             [com.puppetlabs.puppetdb.command.dlo :as dlo]
             [com.puppetlabs.mq :as mq]
             [com.puppetlabs.utils :as pl-utils]
@@ -73,6 +73,7 @@
         [cheshire.custom :only (JSONable)]
         [clj-http.util :only [url-encode]]
         [com.puppetlabs.jdbc :only (with-transacted-connection)]
+        [com.puppetlabs.puppetdb.command.constants :only [command-names]]
         [metrics.meters :only (meter mark!)]
         [metrics.histograms :only (histogram update!)]
         [metrics.timers :only (timer time!)]))
@@ -315,23 +316,30 @@
       ;; Only store a catalog if it's newer than the current catalog
       (if-not (scf-storage/catalog-newer-than? certname timestamp)
         (scf-storage/replace-catalog! catalog timestamp)))
-    (log/info (format "[%s] [replace catalog] %s" id certname))))
+    (log/info (format "[%s] [%s] %s" id (command-names :replace-catalog) certname))))
 
-(defmethod process-command! ["replace catalog" 1]
+(defmethod process-command! [(command-names :replace-catalog) 1]
   [{:keys [version payload] :as command} options]
   {:pre [(= version 1)]}
   (when-not (string? payload)
-    (throw (IllegalArgumentException. "Payload for a 'replace catalog' v1 command must be a JSON string.")))
+    (throw (IllegalArgumentException.
+             (format "Payload for a '%s' v1 command must be a JSON string."
+               (command-names :replace-catalog)))))
   (replace-catalog* command options))
 
-(defmethod process-command! ["replace catalog" 2]
+(defmethod process-command! [(command-names :replace-catalog) 2]
   [{:keys [version] :as  command} options]
   {:pre [(= version 2)]}
   (replace-catalog* command options))
 
+(defmethod process-command! [(command-names :replace-catalog) 3]
+  [{:keys [version] :as  command} options]
+  {:pre [(= version 3)]}
+  (replace-catalog* command options))
+
 ;; Fact replacement
 
-(defmethod process-command! ["replace facts" 1]
+(defmethod process-command! [(command-names :replace-facts) 1]
   [{:keys [payload annotations]} {:keys [db]}]
   (let [{:strs [name] :as facts} (upon-error-throw-fatality (json/parse-string payload))
         id                       (:id annotations)
@@ -340,11 +348,11 @@
       (scf-storage/maybe-activate-node! name timestamp)
       (if-not (scf-storage/facts-newer-than? name timestamp)
         (scf-storage/replace-facts! facts timestamp)))
-    (log/info (format "[%s] [replace facts] %s" id name))))
+    (log/info (format "[%s] [%s] %s" id (command-names :replace-facts) name))))
 
 ;; Node deactivation
 
-(defmethod process-command! ["deactivate node" 1]
+(defmethod process-command! [(command-names :deactivate-node) 1]
   [{:keys [payload annotations]} {:keys [db]}]
   (let [certname (upon-error-throw-fatality (json/parse-string payload))
         id       (:id annotations)]
@@ -352,22 +360,33 @@
       (when-not (scf-storage/certname-exists? certname)
         (scf-storage/add-certname! certname))
       (scf-storage/deactivate-node! certname))
-    (log/info (format "[%s] [deactivate node] %s" id certname))))
+    (log/info (format "[%s] [%s] %s" id (command-names :deactivate-node) certname))))
 
 ;; Report submission
 
-(defmethod process-command! ["store report" 1]
-  [{:keys [payload annotations]} {:keys [db]}]
+(defn store-report*
+  [version db {:keys [payload annotations]}]
   (let [id          (:id annotations)
         report      (upon-error-throw-fatality
-                      (report/validate! payload))
-        name        (:certname report)
+                      (report/validate! version payload))
+        certname    (:certname report)
         timestamp   (:received annotations)]
     (with-transacted-connection db
-      (scf-storage/maybe-activate-node! name timestamp)
+      (scf-storage/maybe-activate-node! certname timestamp)
       (scf-storage/add-report! report timestamp))
-    (log/info (format "[%s] [store report (EXPERIMENTAL!)] puppet v%s - %s"
-                id (:puppet-version report) (:certname report)))))
+    (log/info (format "[%s] [%s] puppet v%s - %s"
+                id (command-names :store-report)
+                (:puppet-version report) (:certname report)))))
+
+(defmethod process-command! [(command-names :store-report) 1]
+  [{:keys [version] :as command} {:keys [db]}]
+  {:pre [(= version 1)]}
+  (store-report* 1 db command))
+
+(defmethod process-command! [(command-names :store-report) 2]
+  [{:keys [version] :as command} {:keys [db]}]
+  {:pre [(= version 2)]}
+  (store-report* 2 db command))
 
 ;; ## MQ I/O
 ;;
