@@ -315,8 +315,59 @@
                (format "Unsupported database engine '%s'"
                  (sql-current-connection-database-name)))))))
 
-;; The available migrations, as a map from migration version to migration
-;; function.
+(defn burgundy-schema-changes
+  "Schema changes for the initial release of Burgundy. These include:
+
+    - Add 'file' and 'line' columns to the event table
+    - A column for the resource's containment path in the resource_events table
+    - A column for the transaction uuid in the reports & catalogs tables
+    - Renames the `sourcefile` and `sourceline` columns on the `catalog_resources`
+      table to `file` and `line` for consistency.
+    - Add index to 'property' column in resource_events table"
+  []
+  (sql/do-commands
+    "ALTER TABLE resource_events ADD COLUMN file VARCHAR(1024) DEFAULT NULL"
+    "ALTER TABLE resource_events ADD COLUMN line INTEGER DEFAULT NULL")
+  (sql/do-commands
+    (format "ALTER TABLE resource_events ADD containment_path %s" (sql-array-type-string "TEXT"))
+    "ALTER TABLE resource_events ADD containing_class VARCHAR(255)"
+    "CREATE INDEX idx_resource_events_containing_class ON resource_events(containing_class)"
+    "CREATE INDEX idx_resource_events_property ON resource_events(property)")
+  (sql/do-commands
+    ;; It would be nice to change the transaction UUID column to NOT NULL in the future
+    ;; once we stop supporting older versions of Puppet that don't have this field.
+    "ALTER TABLE reports ADD COLUMN transaction_uuid VARCHAR(255) DEFAULT NULL"
+    "CREATE INDEX idx_reports_transaction_uuid ON reports(transaction_uuid)"
+    "ALTER TABLE catalogs ADD COLUMN transaction_uuid VARCHAR(255) DEFAULT NULL"
+    "CREATE INDEX idx_catalogs_transaction_uuid ON catalogs(transaction_uuid)")
+  (sql/do-commands
+    (if (= (sql-current-connection-database-name) "PostgreSQL")
+      "ALTER TABLE catalog_resources RENAME COLUMN sourcefile TO file"
+      "ALTER TABLE catalog_resources ALTER COLUMN sourcefile RENAME TO file")
+    (if (= (sql-current-connection-database-name) "PostgreSQL")
+      "ALTER TABLE catalog_resources RENAME COLUMN sourceline TO line"
+      "ALTER TABLE catalog_resources ALTER COLUMN sourceline RENAME TO line")))
+
+(defn add-latest-reports-table
+  "Add `latest_reports` table for easy lookup of latest report for each certname."
+  []
+  (sql/create-table :latest_reports
+    ["certname" "TEXT" "NOT NULL" "PRIMARY KEY" "REFERENCES certnames(name)" "ON DELETE CASCADE"]
+    ["report" "VARCHAR(40)" "NOT NULL" "REFERENCES reports(hash)" "ON DELETE CASCADE"])
+  (sql/do-commands
+    "CREATE INDEX idx_latest_reports_report ON latest_reports(report)")
+  (sql/do-commands
+    "INSERT INTO latest_reports (certname, report)
+        SELECT reports.certname, reports.hash
+        FROM reports INNER JOIN (
+          SELECT reports.certname, MAX(reports.end_time) as max_end_time
+             FROM reports
+             GROUP BY reports.certname
+        ) latest
+          ON reports.certname = latest.certname
+          AND reports.end_time = latest.max_end_time"))
+
+;; The available migrations, as a map from migration version to migration function.
 (def migrations
   {1 initialize-store
    2 allow-node-deactivation
@@ -328,13 +379,15 @@
    8 rename-fact-column
    9 add-reports-tables
    10 add-event-status-index
-   11 increase-puppet-version-field-length})
+   11 increase-puppet-version-field-length
+   12 burgundy-schema-changes
+   13 add-latest-reports-table})
 
 (def desired-schema-version (apply max (keys migrations)))
 
 (defn record-migration!
   "Records a migration by storing its version in the schema_migrations table,
-along with the time at which the migration was performed."
+  along with the time at which the migration was performed."
   [version]
   {:pre [(integer? version)]}
   (sql/do-prepared
@@ -368,7 +421,7 @@ along with the time at which the migration was performed."
 
 (defn migrate!
   "Migrates database to the latest schema version. Does nothing if database is
-already at the latest schema version."
+  already at the latest schema version."
   []
   (if-let [unexpected (first (difference (applied-migrations) (utils/keyset migrations)))]
     (throw (IllegalStateException.
