@@ -19,8 +19,8 @@
 ;; up, it's important to run `garbage-collect!`.
 
 (ns com.puppetlabs.puppetdb.scf.storage
-  (:require [com.puppetlabs.puppetdb.catalog :as cat]
-            [com.puppetlabs.puppetdb.report :as report]
+  (:require [com.puppetlabs.puppetdb.catalogs :as cat]
+            [com.puppetlabs.puppetdb.reports :as report]
             [com.puppetlabs.utils :as utils]
             [com.puppetlabs.jdbc :as jdbc]
             [clojure.java.jdbc :as sql]
@@ -328,24 +328,28 @@ must be supplied as the value to be matched."
 
 (defn add-catalog-metadata!
   "Given some catalog metadata, persist it in the db"
-  [hash api-version catalog-version]
+  [hash api-version catalog-version transaction-uuid]
   {:pre [(string? hash)
          (number? api-version)
-         (string? catalog-version)]}
-  (sql/insert-record :catalogs {:hash            hash
-                                :api_version     api-version
-                                :catalog_version catalog-version}))
+         (string? catalog-version)
+         ((some-fn nil? string?) transaction-uuid)]}
+  (sql/insert-record :catalogs {:hash             hash
+                                :api_version      api-version
+                                :catalog_version  catalog-version
+                                :transaction_uuid transaction-uuid}))
 
 (defn update-catalog-metadata!
   "Given some catalog metadata, update the db"
-  [hash api-version catalog-version]
+  [hash api-version catalog-version transaction-uuid]
   {:pre [(string? hash)
          (number? api-version)
-         (string? catalog-version)]}
+         (string? catalog-version)
+         ((some-fn nil? string?) transaction-uuid)]}
   (sql/update-values :catalogs
                      ["hash=?" hash]
-                     {:api_version     api-version
-                      :catalog_version catalog-version}))
+                     {:api_version      api-version
+                      :catalog_version  catalog-version
+                      :transaction_uuid transaction-uuid}))
 
 (defn catalog-exists?
   "Returns a boolean indicating whether or not the given catalog exists in the db"
@@ -361,7 +365,7 @@ must be supplied as the value to be matched."
   [resource-hashes]
   {:pre  [(coll? resource-hashes)
           (every? string? resource-hashes)]
-   :post [(set? resource-hashes)]}
+   :post [(set? %)]}
   (let [qmarks     (str/join "," (repeat (count resource-hashes) "?"))
         query      (format "SELECT DISTINCT resource FROM resource_params WHERE resource IN (%s)" qmarks)
         sql-params (vec (cons query resource-hashes))]
@@ -433,7 +437,7 @@ must be supplied as the value to be matched."
   The result map has the following format:
 
     {:hashes [[<catalog hash> <resource hash>] ...]
-     :metadata [[<resouce hash> <type> <title> <exported?> <sourcefile> <sourceline>] ...]
+     :metadata [[<resouce hash> <type> <title> <exported?> <file> <line>] ...]
      :parameters [[<resource hash> <name> <value>] ...]
      :tags [[<resource hash> <tag>] ...]}
 
@@ -461,7 +465,7 @@ must be supplied as the value to be matched."
         resource-values (for [[ref resource] refs-to-resources
                               :let [hash (refs-to-hashes ref)]]
                           (resource->values catalog-hash resource hash (persisted? hash)))
-        lookup-table    [[:resource "INSERT INTO catalog_resources (catalog,resource,type,title,tags,exported,sourcefile,sourceline) VALUES (?,?,?,?,?,?,?,?)"]
+        lookup-table    [[:resource "INSERT INTO catalog_resources (catalog,resource,type,title,tags,exported,file,line) VALUES (?,?,?,?,?,?,?,?)"]
                          [:parameters "INSERT INTO resource_params (resource,name,value) VALUES (?,?,?)"]]]
     (sql/transaction
      (doseq [[lookup the-sql] lookup-table
@@ -538,7 +542,7 @@ must be supplied as the value to be matched."
 (defn add-catalog!
   "Persist the supplied catalog in the database, returning its
   similarity hash"
-  [{:keys [api-version version resources edges] :as catalog}]
+  [{:keys [api-version version transaction-uuid resources edges] :as catalog}]
   {:pre [(number? api-version)
          (coll? edges)
          (map? resources)]}
@@ -555,11 +559,11 @@ must be supplied as the value to be matched."
 
               (when exists?
                 (inc! (:duplicate-catalog metrics))
-                (update-catalog-metadata! hash api-version version))
+                (update-catalog-metadata! hash api-version version transaction-uuid))
 
               (when-not exists?
                 (inc! (:new-catalog metrics))
-                (add-catalog-metadata! hash api-version version)
+                (add-catalog-metadata! hash api-version version transaction-uuid)
                 (let [refs-to-hashes (zipmap (keys resources) resource-hashes)]
                   (time! (:add-resources metrics)
                          (add-resources! hash resources refs-to-hashes))
@@ -690,7 +694,7 @@ must be supplied as the value to be matched."
   property, values, status, timestamp, etc.)
   "
   [{:keys [resource-type resource-title property timestamp status old-value
-           new-value message] :as resource-event}]
+           new-value message file line] :as resource-event}]
   (-> (sort { :resource-type resource-type
               :resource-title resource-title
               :property property
@@ -698,7 +702,9 @@ must be supplied as the value to be matched."
               :status status
               :old-value old-value
               :new-value new-value
-              :message message})
+              :message message
+              :file file
+              :line line})
       (pr-str)
       (utils/utf8-string->sha1)))
 
@@ -710,7 +716,7 @@ must be supplied as the value to be matched."
   configuration version, timestamps, events).
   "
   [{:keys [certname puppet-version report-format configuration-version
-           start-time end-time resource-events] :as report}]
+           start-time end-time resource-events transaction-uuid] :as report}]
   (-> (sorted-map)
     (assoc :certname certname)
     (assoc :puppet-version puppet-version)
@@ -719,29 +725,67 @@ must be supplied as the value to be matched."
     (assoc :start-time start-time)
     (assoc :end-time end-time)
     (assoc :resource-events (sort (map resource-event-identity-string resource-events)))
+    (assoc :transaction-uuid transaction-uuid)
     (pr-str)
     (utils/utf8-string->sha1)))
 
-(defn add-report!
-  "Add a report and all of the associated events to the database."
+(defn update-latest-report!
+  "Given a node name, updates the `latest_reports` table to ensure that it indicates the
+  most recent report for the node."
+  [node]
+  {:pre [(string? node)]}
+  (let [latest-report (:hash (first (query-to-vec
+                                      ["SELECT hash FROM reports
+                                            WHERE certname = ?
+                                            ORDER BY end_time DESC
+                                            LIMIT 1" node])))]
+    (sql/update-or-insert-values
+      :latest_reports
+      ["certname = ?" node]
+      {:certname      node
+       :report        latest-report})))
+
+(defn find-containing-class
+  "Given a containment path from Puppet, find the outermost 'class'."
+  [containment-path]
+  {:pre [(or
+           (nil? containment-path)
+           (and (coll? containment-path) (every? string? containment-path)))]
+   :post [((some-fn nil? string?) %)]}
+  (when-not ((some-fn nil? empty?) containment-path)
+    ;; This is a little wonky.  Puppet only gives us an array of Strings
+    ;; to represent the containment path.  Classes can be differentiated
+    ;; from types because types have square brackets and a title; so, e.g.,
+    ;; "Foo" is a class, but "Foo[Bar]" is a type with a title.
+    (first
+      (filter
+        #(not (or (empty? %) (utils/string-contains? "[" %)))
+        (reverse containment-path)))))
+
+(defn add-report!*
+  "Helper function for adding a report.  Accepts an extra parameter, `update-latest-report?`, which
+  is used to determine whether or not the `update-latest-report!` function will be called as part of
+  the transaction.  This should always be set to `true`, except during some very specific testing
+  scenarios."
   [{:keys [puppet-version certname report-format configuration-version
-           start-time end-time resource-events]
+           start-time end-time resource-events transaction-uuid]
     :as report}
-   timestamp]
+   timestamp update-latest-report?]
   {:pre [(map? report)
-         (utils/datetime? timestamp)]}
+         (utils/datetime? timestamp)
+         (utils/boolean? update-latest-report?)]}
   (let [report-hash         (report-identity-string report)
+        containment-path-fn (fn [cp] (if-not (nil? cp) (to-jdbc-varchar-array cp)))
         resource-event-rows (map #(-> %
                                      (update-in [:timestamp] to-timestamp)
                                      (update-in [:old-value] db-serialize)
                                      (update-in [:new-value] db-serialize)
-                                     (assoc :report report-hash)
-                                     ((partial utils/mapkeys dashes->underscores)))
+                                     (update-in [:containment-path] containment-path-fn)
+                                     (assoc :containing-class (find-containing-class (% :containment-path)))
+                                     (assoc :report report-hash) ((partial utils/mapkeys dashes->underscores)))
                                   resource-events)]
     (time! (:store-report metrics)
       (sql/transaction
-        ;; TODO: should probably do some checking / error-handling around
-        ;; whether or not the report id already exists
         (sql/insert-record :reports
           { :hash                   report-hash
             :puppet_version         puppet-version
@@ -750,8 +794,16 @@ must be supplied as the value to be matched."
             :configuration_version  configuration-version
             :start_time             (to-timestamp start-time)
             :end_time               (to-timestamp end-time)
-            :receive_time           (to-timestamp timestamp)})
-        (apply sql/insert-records :resource_events resource-event-rows)))))
+            :receive_time           (to-timestamp timestamp)
+            :transaction_uuid       transaction-uuid})
+        (apply sql/insert-records :resource_events resource-event-rows)
+        (if update-latest-report?
+          (update-latest-report! certname))))))
+
+(defn add-report!
+  "Add a report and all of the associated events to the database."
+  [report timestamp]
+  (add-report!* report timestamp true))
 
 (defn delete-reports-older-than!
   "Delete all reports in the database which have an `end-time` that is prior to
